/*
    icmp.c: icmp log part of Tcp Listen
    Copyright (C) 1999 by Marcelo Gornstein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    see "COPYING" file for details.

If you want to contact me:
Marcelo Gornstein
mgornstein@usa.net
*/

#define D_POSIX_SOURCE
#include <stdio.h>
#include <netinet/ip.h>
#include <netinet/udp.h>
#include <netinet/tcp.h>
#include <netinet/ip_icmp.h>

#include <syslog.h>

int log_icmp (struct icmphdr *icmp, struct iphdr *ip, struct in_addr add,
						 char syslogg, char *buffer)
{
char logstring [256];
char aux [256];

memset (logstring, '\0', sizeof (logstring));
memset (aux, '\0', sizeof (aux));

add.s_addr = ip->saddr;
sprintf (logstring, "ICMP: %s > ", inet_ntoa (add.s_addr));
add.s_addr = ip->daddr;
sprintf (aux, "%s ", inet_ntoa (add.s_addr));
strcat (logstring, aux);
memset (aux, '\0', sizeof (aux));

if (icmp->type == ICMP_ECHO) sprintf (aux, "echo request. id = %d seq = %d.", 
							icmp->un.echo.id, 
 							icmp->un.echo.sequence);

if (icmp->type == ICMP_ECHOREPLY) sprintf (aux, "echo reply. id = %d seq = %d.", 
							icmp->un.echo.id, 
 							icmp->un.echo.sequence);

if (icmp->type == ICMP_SOURCE_QUENCH) sprintf (aux, "source quench");

if (icmp->type == ICMP_REDIRECT) {
	if (icmp->code == 0) sprintf (aux, "redirect net ");
	if (icmp->code == 1) sprintf (aux, "redirect host ");
	if (icmp->code == 2) sprintf (aux, "redirect net for TOS ");
	if (icmp->code == 3) sprintf (aux, "redirect host for TOS ");
				}

if (icmp->type == ICMP_TIME_EXCEEDED) {
	if (icmp->code == 0) sprintf (aux, "ttl exceeded ");
	if (icmp->code == 1) sprintf (aux, "frag reas exceeded ");
				}

if (icmp->type == ICMP_PARAMETERPROB) sprintf (aux, "parameter problem ");

if (icmp->type == ICMP_TIMESTAMP) sprintf (aux, "timestamp request ");
if (icmp->type == ICMP_TIMESTAMPREPLY) sprintf (aux, "timstamp reply ");

if (icmp->type == ICMP_INFO_REQUEST) sprintf (aux, "info request ");
if (icmp->type == ICMP_INFO_REPLY) sprintf (aux, "info reply ");
if (icmp->type == ICMP_ADDRESS) sprintf (aux, "netmask request ");
if (icmp->type == ICMP_ADDRESSREPLY) sprintf (aux, "netmask reply ");

if (icmp->type == ICMP_DEST_UNREACH) {
	if (icmp->code == 0) sprintf (aux, "net unrecheable ");
	if (icmp->code == 1) sprintf (aux, "host unracheable ");
	if (icmp->code == 2) sprintf (aux, "protocol unrecheable ");
	if (icmp->code == 3) sprintf (aux, "port unrecheable ");
	if (icmp->code == 4) sprintf (aux, "frag needed ");
	if (icmp->code == 5) sprintf (aux, "src route failed ");
	if (icmp->code == 6) sprintf (aux, "target net unknown ");
	if (icmp->code == 7) sprintf (aux, "target host unknown");
	if (icmp->code == 8) sprintf (aux, "src host isolated ");
	if (icmp->code == 9) sprintf (aux, "net admin prohibited ");
	if (icmp->code == 10) sprintf (aux, "host admin prohibited ");
	if (icmp->code == 11) sprintf (aux, "net unrecheable by TOS");
	if (icmp->code == 12) sprintf (aux, "host unracheable by TOS");
	}

strcat (logstring, aux);
strcat (logstring, "\n");
if (!syslogg) printf ("%s", logstring); else syslog (LOG_INFO, logstring);
return (0);
}
