/*
    main.c: main part of Tcp Listen
    Copyright (C) 1999 by Marcelo Gornstein

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    see "COPYING" file for details.

If you want to contact me:
Marcelo Gornstein
mgornstein@usa.net
*/

#define D_POSIX_SOURCE

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/ethernet.h>
#include <net/if.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/ip_icmp.h>
#include <signal.h>
#include <syslog.h>

#define iplen sizeof (struct iphdr)
#define tcplen sizeof (struct tcphdr)
#define udplen sizeof (struct udphdr)
#define icmplen sizeof (struct icmphdr)
#define ethlen sizeof (struct ethhdr)

void watchpackets (void);
int main (int argc, char *argv []);
void cleanup (int sig);

char logtcp = 1;
char logudp = 1;
char logicmp = 1;
char syslogg = 0;
char forked = 0;
char buffer [2048];

struct sockaddr_in pck_info;
struct ethhdr *eth = (struct ethhdr *) (buffer);
struct iphdr *ip = (struct iphdr *) (buffer + ethlen);
struct tcphdr *tcp = (struct tcphdr *) (buffer + ethlen + iplen);
struct udphdr *udp = (struct udphdr *) (buffer + ethlen + iplen);
struct icmphdr *icmp = (struct icmphdr *) (buffer + ethlen + iplen);
struct in_addr add;
int r;
int s;
int i;

void watchpackets (void)
{

while (1)
{
r = recvfrom (s, buffer, sizeof (buffer), 0, (struct sockaddr*)(&pck_info), &r);
if (logtcp) if (ip->protocol == 6) log_tcp (tcp, ip, add, syslogg);
if (logudp) if (ip->protocol == 17) log_udp (udp, ip, add, syslogg);
if (logicmp) if (ip->protocol == 1) log_icmp (icmp, ip, add, syslogg, buffer);
ip = (struct iphdr *) (buffer);
tcp = (struct tcphdr *) (buffer + iplen);
udp = (struct udphdr *) (buffer + iplen);
icmp = (struct icmphdr *) (buffer + iplen);
if (logtcp) if (ip->protocol == 6) log_tcp (tcp, ip, add, syslogg);
if (logudp) if (ip->protocol == 17) log_udp (udp, ip, add, syslogg);
if (logicmp) if (ip->protocol == 1) log_icmp (icmp, ip, add, syslogg, buffer);
ip = (struct iphdr *) (buffer + ethlen);
tcp = (struct tcphdr *) (buffer + ethlen + iplen);
udp = (struct udphdr *) (buffer + ethlen + iplen);
icmp = (struct icmphdr *) (buffer + ethlen + iplen);
memset (buffer, '\0', sizeof (buffer));
}
}

int main (int argc, char *argv [])
{
memset (buffer, '\0', sizeof (buffer));
if (syslogg) openlog (argv [0], LOG_NDELAY|LOG_PERROR, LOG_USER);

if (argc > 1)
{
for (i = 1; i < argc; i++)
	{
	if (!strcmp (argv [i], "-notcp")) logtcp = 0;
	if (!strcmp (argv [i], "-noudp")) logudp = 0;
	if (!strcmp (argv [i], "-noicmp")) logicmp = 0;
	if (!strcmp (argv [i], "-tcp")) logtcp = 1;
	if (!strcmp (argv [i], "-udp")) logudp = 1;
	if (!strcmp (argv [i], "-icmp")) logicmp = 1;
	if (!strcmp (argv [i], "-syslog")) syslogg = 1;
	if (!strcmp (argv [i], "-fork")) forked = 1;
	}
}

s = socket (AF_INET, SOCK_PACKET, htons (ETHERTYPE_IP));
if (s < 0) { perror ("Error"); exit (-1); }

r = sizeof (struct sockaddr);

signal (SIGSEGV, cleanup);
signal (SIGTERM, cleanup);
signal (SIGQUIT, cleanup);
signal (SIGKILL, cleanup);
signal (SIGINT, cleanup);
signal (SIGHUP, cleanup);

if (forked) { if (!fork ()) watchpackets (); } else watchpackets ();

exit (0);
}

void cleanup (int sig)
{
printf ("Caught signal!\n");
if (syslogg) { syslog (LOG_INFO, "exiting."); closelog (); }
close (s);
exit (0);
}
